import flask_login as fl
from flask import Blueprint, flash, redirect, render_template, request, url_for, current_app

from yadc import db
from yadc.forms import LoginForm, RegisterForm, ResetPasswordForm, ResetPasswordPassForm
from yadc.models import User
from yadc.utils import nextpage, flasherrors

bp = Blueprint('auth', __name__)

@bp.route('/login', methods=['GET', 'POST'])
def login():
    if fl.current_user.is_authenticated:
        return redirect(url_for('main.index'))

    form = LoginForm(request.form)
    if request.method == 'POST' and form.validate():
        user = User.query.filter_by(username=form.username.data).first()
        if user is None or not user.check_password(form.password.data):
            flash('Invalid username or password.', category='error')
            return redirect(url_for('.login'))

        user.login(remember=form.remember_me.data)
        db.session.commit()

        flash(f'Logged in as {user.username}.')

        return redirect(nextpage())

    flasherrors(form)
    return render_template('auth/login.html', form=form)

@bp.route('/logout')
def logout():
    if fl.current_user.is_authenticated:
        fl.current_user.logout()

        flash('Successfully logged out.')
    return redirect(nextpage())

@bp.route('/reset_password', methods=['GET', 'POST'])
def reset_password():
    if fl.current_user.is_authenticated:
        return redirect(url_for('main.index'))

    form = ResetPasswordPassForm(request.form)
    if request.method == 'POST' and form.validate():
        user = User.query.filter_by(email=form.email.data).first()
        user.create_password(form.password.data)
        db.session.commit()

        flash('Password successfully reset.') # for real
        return redirect(url_for('.login'))

    form = ResetPasswordForm(request.form)
    if request.method == 'POST' and form.validate():
        user = User.query.filter_by(email=form.email.data).first()

        if user:
            return render_template('auth/reset_password.html', form=ResetPasswordPassForm(request.form))

        flash('Password successfully reset.') # faked
        return redirect(url_for('.login'))

    flasherrors(form)
    return render_template('auth/reset_password.html', form=form)

@bp.route('/register', methods=['GET', 'POST'])
def register():
    if fl.current_user.is_authenticated:
        return redirect(url_for('main.index'))

    if current_app.config.get('REGISTER_DISABLED'):
        flash('Registrations are disabled for now.')
        return redirect(url_for('main.index'))

    form = RegisterForm(request.form)
    if request.method == 'POST' and form.validate():
        user = User(username=form.username.data, email=form.email.data)
        user.create_password(form.password.data)

        db.session.add(user)
        db.session.commit()
        
        flash('Your account has been successfully registered. You can now login.')
        return redirect(url_for('.login'))

    flasherrors(form)
    return render_template('auth/register.html', form=form)
