from flask import (Blueprint, current_app, flash, redirect,
                   render_template, request, send_from_directory, url_for)
from flask_login import login_required, current_user
from yadc.forms import UserForm, PostForm, TagForm, CommentForm

from yadc import db
from yadc.models import User, USER_STATUS, moderator_required, admin_required, Post, Tag, TAG_CATEGORY, Comment, POST_STATUS
from yadc.utils import flasherrors

bp = Blueprint('manage', __name__)

@bp.route('/users', defaults={'page': 1})
@bp.route('/users/<int:page>')
@login_required
@admin_required
def manage_users(page):
    users = User.query.order_by(User.id.desc()).paginate(page, current_app.config.get('MANAGE_PER_PAGE'))

    for user in users.items:
        user.editform = UserForm(
            id=user.id,
            username=user.username,
            email=user.email,
            op_level=user.op_level.name,
            user_status=user.user_status.name
        )

    return render_template('manage/users.html', users=users, elements=users.items, createform=UserForm())

@bp.route('/posts', defaults={'page': 1})
@bp.route('/posts/<int:page>')
@login_required
def manage_posts(page):
    if current_user.is_moderator:
        posts = Post.query.order_by(Post.id.desc()).paginate(page, current_app.config.get('MANAGE_PER_PAGE'))
    else:
        posts = Post.query.filter_by(author=current_user).order_by(Post.id.desc()).paginate(page, current_app.config.get('MANAGE_PER_PAGE'))

    for post in posts.items:
        post.editform = PostForm(
            id=post.id,
            rating=post.rating.name,
            status=post.status.name,
            source=post.source,
            parent=post.parent_id
        )

    return render_template('manage/posts.html', posts=posts, elements=posts.items)

@bp.route('/tags', defaults={'page': 1})
@bp.route('/tags/<int:page>')
@login_required
@moderator_required
def manage_tags(page):
    tags = Tag.query.order_by(Tag.content).paginate(page, current_app.config.get('MANAGE_PER_PAGE'))

    for tag in tags.items:
        tag.editform = TagForm(
            id=tag.id,
            content=tag.content_deser,
            category=tag.category.name
        )

    return render_template('manage/tags.html', tags=tags, elements=tags.items, createform=TagForm())

# ONLY THROUGH MANAGEMENT
@bp.route('/modify_user', methods=['POST'])
@login_required
@admin_required
def modify_user():
    form = UserForm(request.form)
    # flash(str(request.form))
    if form.validate():
        if form.create.data:
            el = User(username=form.username.data)
            db.session.add(el)

            if form.email.data: el.email = form.email.data
            if form.user_status.data: el.user_status = form.user_status.data
            if form.op_level.data: el.op_level = form.op_level.data

            db.session.commit()
            #flash('New {} has been created.'.format(str(el)))
            flash('New user has been created.')

        else:
            el = User.query.get(form.id.data)

            if form.delete.data:
                if el.is_current:
                    flash("Your account can be deleted from profile settings page.", category='error')
                    return redirect(url_for('.manage_users'))
                elif el.is_admin:
                    flash("You can't delete admins.", category='error')
                    return redirect(url_for('.manage_users'))

                db.session.delete(el)
                db.session.commit()
                #flash('{} deleted.'.format(str(el)))
                flash('User deleted.')

            elif form.edit.data:
                # if form.username.data: el.username = form.username.data

                if form.email.raw_data and form.email.data: el.email = form.email.data
                if form.user_status.raw_data and form.user_status.data: el.user_status = form.user_status.data
                if form.op_level.raw_data and form.op_level.data: el.op_level = form.op_level.data

                db.session.commit()
                #flash('Changes to {} have been applied.'.format(str(el)))
                flash('Changes to user have been applied.')

    flasherrors(form)
    return redirect(url_for('.manage_users'))

# THROUGH MANAGEMENT AND POST PAGE + USERS CAN USE
@bp.route('/modify_post', methods=['POST'])
@login_required
def modify_post():
    form = PostForm(request.form)
    # flash(str(request.form))
    if form.validate():
        if form.create.data:
            pass
        else:
            el = Post.query.get(form.id.data)
            if not current_user.is_moderator and not (el.author.is_current if el.author is not None else None):
                flash("You don't have sufficient rights to do this.")
                return redirect(url_for('main.index'))

            if form.delete.data:
                el.remove_image_files()
                db.session.delete(el)
                db.session.commit()
                #flash('{} deleted.'.format(str(el)))
                flash('Post deleted.')

            elif form.edit.data:
                if form.rating.raw_data and form.rating.data: el.rating = form.rating.data
                if form.status.raw_data and form.status.data: el.status = form.status.data
                if form.source.raw_data: el.source = form.source.data

                if form.parent.raw_data:
                    el.parent_id = form.parent.data
                    # el.parent = Post.query.get(form.parent.data)

                if form.tags.raw_data and form.tags.data:
                    f_tags = form.tags.data.split()
                    tags = Tag.query.filter(Tag.content.in_(f_tags)).all()
                    el.tags = tags

                db.session.commit()
                #flash('Changes to {} have been applied.'.format(str(el)))
                flash('Changes to post have been applied.')

            elif form.approve.data:
                if not current_user.is_moderator:
                    flash("You don't have sufficient rights to do this.")
                    return redirect(url_for('main.index'))
                el.status = POST_STATUS.active
                el.approver = current_user

                db.session.commit()
                #flash('Approved post {}.'.format(str(el)))
                flash('Post approved.')
                # redirect(url_for('post.post_show', id=el.id))

    flasherrors(form)
    if form.referer.data == 'post_show':
        return redirect(url_for('post.post_show', id=form.id.data))

    return redirect(url_for('.manage_posts'))

# Example perfect create/edit/delete form endpoint
# ONLY THROUGH MANAGEMENT
@bp.route('/modify_tag', methods=['POST'])
@login_required
@moderator_required
def modify_tag():
    form = TagForm(request.form)
    # flash(str(request.form))
    if form.validate():
        if form.create.data:
            el = Tag(content_deser=form.content.data.lower())
            db.session.add(el)

            if form.category.raw_data and form.category.data: el.category = form.category.data

            db.session.commit()
            #flash('New {} has been created.'.format(str(el)))
            flash('New tag has been created.')
        else:
            el = Tag.query.get(form.id.data)

            if form.delete.data:
                db.session.delete(el)
                db.session.commit()
                #flash('{} deleted.'.format(str(el)))
                flash('Tag deleted.')

            elif form.edit.data:
                # if form.content.data: el.content = form.content.data
                if form.category.raw_data and form.category.data: el.category = form.category.data

                db.session.commit()
                #flash('Changes to {} have been applied.'.format(str(el)))
                flash('Changes to tag have been applied.')

    flasherrors(form)
    return redirect(url_for('.manage_tags'))
    # return redirect(url_for('main.index'))

@bp.route('/modify_comment', methods=['POST'])
@login_required
@moderator_required
def modify_comment():
    form = CommentForm(request.form)
    # flash(str(request.form))
    if form.validate():
        if form.create.data:
            pass
        else:
            el = Comment.query.get(form.id.data)

            if form.delete.data:
                db.session.delete(el)
                db.session.commit()
                #flash('{} deleted.'.format(str(el)))
                flash('Comment deleted.')

            elif form.edit.data:
                # if form.rating.raw_data and form.rating.data: el.rating = form.rating.data
                # if form.status.raw_data and form.status.data: el.status = form.status.data
                # if form.source.raw_data: el.source = form.source.data

                # db.session.commit()
                #flash('Changes to {} have been applied.'.format(str(el)))
                flash('Changes to comment have been applied.')

            elif form.ban.data:
                el.ban_reason = form.ban_reason.data
                el.banned = True

                db.session.commit()
                #flash('Comment {} has been banned.'.format(str(el)))
                flash('Comment has been banned.')

            elif form.unban.data:
                el.ban_reason = None
                el.banned = False

                db.session.commit()
                #flash('Comment {} has been unbanned.'.format(str(el)))
                flash('Comment has been unbanned.')

    flasherrors(form)
    if form.referer.data == 'post_show':
        return redirect(url_for('post.post_show', id=Comment.query.get(form.id.data).post_id))

    return redirect(url_for('.manage_comments'))