import io
import os

from flask import (Blueprint, abort, current_app, flash, redirect,
                   render_template, request, send_from_directory, url_for, session, jsonify)
from flask_login import current_user, login_required
from sqlalchemy import func
from sqlalchemy.orm import aliased

from yadc import db
from yadc.forms import UploadForm, CommentForm, PostForm
from yadc.models import FILETYPE, RATING, POST_STATUS, IMAGE_STORE, TAG_CATEGORY, Post, Tag, Comment, moderator_required
from yadc.utils import query_replace, flasherrors, session_rating

bp = Blueprint('post', __name__)

@bp.route('', defaults={'page': 1})
@bp.route('/<int:page>')
def posts(page):
    def tags_prepare(posts):
        tags = db.session.query(Tag, func.count(Post.id), func.array_agg(Post.id)).join(Tag.posts).filter(Post.id.in_([p.id for p in posts])).group_by(Tag.id).all()
        # had to do this bc otherwise the index is overflowing with tags
        tags = [tag for tag, tag.count, tag.post_ids in tags if not (tag.count<2 and tag.category is TAG_CATEGORY.general)]
        tags.sort(key=lambda x: x.content)
        tags.sort(key=lambda x: x.count, reverse=True)
        tags.sort(key=lambda x: x.category.value, reverse=True)
        return tags

    # PARSING ARGUMENTS
    if request.args.get('tags') == '':
        return redirect(query_replace({}, url_for('.posts')))
    f_tags = request.args.get('tags', '').split()

    f_rating = {r.name : r for r in RATING}.get(request.args.get('rating'))
    session['enforced_rating'] = f_rating.name if f_rating else None

    if current_user.is_authenticated and current_user.rating is RATING.explicit and f_rating is RATING.explicit:
        return redirect(url_for('.posts')) # Fookin hack to fix the state after user's switch to RATING.explicit

    if current_user.is_authenticated and f_rating is None:
        f_rating = current_user.rating
    m_ratings = f_rating.matched if f_rating else RATING.safe.matched

    # filter user's blacklist
    blacklist_tags = [tb.content for tb in current_user.tag_blacklist] if current_user.is_authenticated else None

    posts_query = Post.query
    if f_tags or blacklist_tags:
        posts_query = posts_query.join(Post.tags).group_by(Post.id)
        if blacklist_tags:
            subquery = db.session.query(Post.id).join(Tag.posts).filter(Tag.content.in_(blacklist_tags)).subquery()
            posts_query = posts_query.filter(~Post.id.in_(subquery))
        if f_tags:
            posts_query = posts_query.filter(Tag.content.in_(f_tags)).having(func.count(Post.id)==len(f_tags))
    if current_user.is_anonymous or not current_user.is_moderator:
        posts_query = posts_query.filter_by(status=POST_STATUS.active)
    posts_query = posts_query.filter(Post.rating.in_(m_ratings)).order_by(Post.created.desc())

    posts = posts_query.paginate(page, current_app.config.get('POSTS_PER_PAGE'))
    tags = tags_prepare(posts.items)
    return render_template(
        'post/index.html',
        posts=posts,
        # posts=[p for p in posts if p.status is POST_STATUS.active] if current_user.is_anonymous or not current_user.is_moderator else posts,
        tags=tags,
        sel_tags=[tag for tag in tags if tag.content in f_tags] or Tag.query.filter(Tag.content.in_(f_tags))
    )

@bp.route('/show/<int:id>')
def post_show(id):
    post = Post.query.get_or_404(id)

    tags_count = db.session.query(Tag, func.count(Post.id)).join(Tag.posts).filter(Post.id==id).join(aliased(Post), Tag.posts).group_by(Tag).all()

    tags = [tag for tag,tag.count in tags_count]
    # ikr, DRY, but I don't have much time anyway, I should have just dropped the no-js support altogether
    tags.sort(key=lambda x: x.content)
    tags.sort(key=lambda x: x.count, reverse=True)
    tags.sort(key=lambda x: x.category.value, reverse=True)

    comments = db.session.query(Comment).filter_by(post=post).filter_by(visible=True).order_by(Comment.id).all()

    for comment in comments:
        comment.editform = CommentForm(id=comment.id, content=comment.content)

    return render_template(
        'post/post.html',
        post=post,
        tags=tags,
        comments=comments,
        editform=PostForm(
            id=post.id,
            referer=post_show.__name__,
            source=post.source,
            parent=post.parent_id,
            tags=' '.join(t.content for t in tags),
            rating=post.rating,
            status=post.status
        ),
        comment_form=CommentForm(post_id=post.id)
    )

@bp.route('/comment', methods=['POST'])
@login_required
def comment():
    form = CommentForm(request.form)
    #flash(str(request.form))
    if form.validate():
        if form.create.data:
            el = Comment(content=form.content.data.strip(), post_id=form.post_id.data, user=current_user)
            db.session.add(el)

            db.session.commit()
            #flash('Successfully submitted {}.'.format(str(el)))
            flash('Comment successfully submitted.')
            return redirect(url_for('post.post_show', id=form.post_id.data))
        else:
            el = Comment.query.filter_by(id=form.id.data).first()

            if form.delete.data:
                db.session.delete(el)
                db.session.commit()
                #flash('{} deleted.'.format(str(el)))
                flash('Comment deleted.')
            elif form.edit.data:
                if form.content.raw_data and form.content.data: el.content = form.content.data.strip()

                db.session.commit()
                #flash('Changes to {} have been applied.'.format(str(el)))
                flash('Changes to comment have been applied.')

            elif form.ban.data:
                if not current_user.is_moderator:
                    flash("You don't have sufficient rights to do this.")
                    return redirect(url_for('main.index'))

                el.deleted = True

                db.session.commit()
                #flash('Comment {} has been banned.'.format(str(el)))
                flash('Comment has been banned.')

            elif form.unban.data:
                if not current_user.is_moderator:
                    flash("You don't have sufficient rights to do this.")
                    return redirect(url_for('main.index'))

                el.deleted = False

                db.session.commit()
                #flash('Comment {} has been unbanned.'.format(str(el)))
                flash('Comment has been unbanned.')

            return redirect(url_for('post.post_show', id=el.post_id))

    flasherrors(form)
    return redirect(url_for('main.index'))

@bp.route('/upload', methods=['GET', 'POST'])
@login_required
def upload():
    form = UploadForm(request.form)
    if request.method == 'POST' and form.validate():
        file = request.files.get(form.post_img.name)

        f_tags = form.tags.data.split()
        tags = Tag.query.filter(Tag.content.in_(f_tags)).all()

        post = Post(**Post.fileinfo(file), source=form.sauce.data, tags=tags, rating=form.rating.data, author=current_user)
        file.seek(0)
        file.save(post.image_files['image'])

        post.generate_image_files()

        db.session.add(post)
        db.session.commit()

        #flash('Successfully submitted {}.'.format(str(post)))
        flash('Upload successfully submitted.')

        return redirect(url_for('.posts'))

    flasherrors(form)
    return render_template('post/upload.html', form=form)

@bp.route('/tags')
def tag_autocomplete():
    query = request.args.get('q', '')
    if query != '':
        tags = Tag.query.filter(Tag.content.like("%{}%".format(query))).limit(15).all()

        return jsonify([{"id": t.id, "content": t.content, "content_deser": t.content_deser, "category": {"id": t.category.value, "name": t.category.name}} for t in tags])
    return jsonify()

# A TRY TO MAKE A DANBOORU COMPATIBLE API
@bp.route('/index.json')
def posts_api():
    f_tags = request.args.get('tags', '').split()
    
    # no need to filter rating, app filters the content itself (even though sends rating pref)
    for t in f_tags:
        if t.startswith('rating:') or t.startswith('-rating:'):
            f_tags.remove(t)

    posts_query = Post.query
    if f_tags:
        posts_query = posts_query.join(Post.tags).group_by(Post.id).filter(Tag.content.in_(f_tags)).having(func.count(Post.id)==len(f_tags))
    posts_query = posts_query.filter_by(status=POST_STATUS.active).order_by(Post.created.desc())

    posts = posts_query.paginate(int(request.args.get('page', 0))+1, int(request.args.get('limit') or current_app.config.get('POSTS_PER_PAGE')))

    return jsonify([dict(
        id=p.id,
        tags=" ".join([t.content for t in p.tags]),
        created_at=int(p.created.timestamp()),
        creator_id=p.author_id,
        source=p.source,
        md5=p.md5,
        file_size=p.filesize,
        file_url=p.url(path=p.file_uri, endpoint=IMAGE_STORE.image),
        preview_url=p.url(path=p.file_uri, endpoint=IMAGE_STORE.thumb_big),

        preview_width=0,
        preview_height=0,
        actual_preview_width=0,
        actual_preview_height=0,

        sample_url=p.url(path=p.file_uri, endpoint=IMAGE_STORE.sample),

        sample_width=0,
        sample_height=0,
        sample_file_size=0,

        jpeg_url=p.url(path=p.file_uri, endpoint=(IMAGE_STORE.image if p.filetype.is_jpeg else IMAGE_STORE.jpeg)),

        jpeg_width=0,
        jpeg_height=0,
        jpeg_file_size=0,

        rating=str(p.rating.name)[:1],

        has_children=False,
        parent_id=None,

        status=p.status.name,
        width=p.width,
        height=p.height,

        is_held=False,
        frames_pending_string='',
        frames_pending=[],
        frames_string='',
        frames=[]
    ) for p in posts.items])