from flask import (Blueprint, abort, current_app, flash, redirect,
                   render_template, request, send_from_directory, url_for)
from flask_login import current_user, login_required
from yadc.forms import ChangeUserInfoForm, ChangePassForm, ChangeMailForm, ChangeUserRatingForm, ChangeTagBlacklistForm, DeleteUserDataForm, CommentForm

from yadc import db
from yadc.models import User, Post, Comment, Tag
from yadc.utils import flasherrors

bp = Blueprint('user', __name__)

@bp.route('/@<username>')
def profile(username):
    user = User.query.filter_by(username=username).first_or_404()
    last_comments = Comment.query.filter_by(user=user).order_by(Comment.id).limit(5).all()

    for comment in last_comments:
        comment.editform = CommentForm(id=comment.id, content=comment.content)

    return render_template(
        'user/profile.html',
        user=user,
        comments=last_comments,
        post_count=Post.query.filter_by(author=user).count(),
        comment_count=Comment.query.filter_by(user=user).count(),
        approved_count=Post.query.filter_by(approver=user).count()
    )

@bp.route('/settings')
@login_required
def settings():
    return render_template(
        'user/settings.html',
        userinfo_form=ChangeUserInfoForm(bio=current_user.biography),
        pass_form=ChangePassForm(),
        mail_form=ChangeMailForm(),
        rating_form=ChangeUserRatingForm(rating=current_user.rating.name),
        tags_form=ChangeTagBlacklistForm(tags=' '.join(t.content for t in current_user.tag_blacklist)), tags_form_tags=current_user.tag_blacklist,
        delete_form=DeleteUserDataForm()
    )

@bp.route('/change_info', methods=['POST'])
@login_required
def change_info():
    form = ChangeUserInfoForm(request.form)
    if form.validate():
        current_user.biography = form.bio.data.strip()

        db.session.commit()
        flash('Your biography was updated.')

    flasherrors(form)
    return redirect(url_for('.settings'))

@bp.route('/change_pass', methods=['POST'])
@login_required
def change_pass():
    form = ChangePassForm(request.form)
    if form.validate():
        if not current_user.check_password(form.password_current.data):
            flash('Incorrect password.', category='error')
            return redirect(url_for('.settings'))

        current_user.create_password(form.password.data)

        db.session.commit()
        flash('Password changed successfully.')

    flasherrors(form)
    return redirect(url_for('.settings'))

@bp.route('/change_mail', methods=['POST'])
@login_required
def change_mail():
    form = ChangeMailForm(request.form)
    if form.validate():
        current_user.email = form.email.data

        db.session.commit()
        flash('Your email was updated.')

    flasherrors(form)
    return redirect(url_for('.settings'))

@bp.route('/change_rating', methods=['POST'])
@login_required
def change_rating():
    form = ChangeUserRatingForm(request.form)
    if form.validate():
        current_user.rating = form.rating.data

        db.session.commit()
        flash('Your rating preference was updated.')

    flasherrors(form)
    return redirect(url_for('.settings'))

@bp.route('/change_tagblacklist', methods=['POST'])
@login_required
def change_tagblacklist():
    form = ChangeTagBlacklistForm(request.form)
    if form.validate():
        f_tags = form.tags.data.split()
        tags = Tag.query.filter(Tag.content.in_(f_tags)).all()
        current_user.tag_blacklist = tags

        db.session.commit()
        flash('Your tag blacklist was updated.')

    flasherrors(form)
    return redirect(url_for('.settings'))

@bp.route('/delete_data', methods=['POST'])
@login_required
def delete_data():
    form = DeleteUserDataForm(request.form)
    if form.validate():
        if form.all_posts:
            Post.query.filter_by(author_id=current_user.id).delete()

        if form.all_comments:
            Comment.query.filter_by(user_id=current_user.id).delete()

        db.session.delete(current_user)
        db.session.commit()
        flash('Thank you for using our service.')

        return redirect(url_for('main.index'))

    flasherrors(form)
    return redirect(url_for('.settings'))