import enum
import hashlib
import humanize
import os
from datetime import datetime, timezone
from functools import wraps

from flask import current_app, url_for, flash, redirect
from flask_login import UserMixin, login_user, logout_user, current_user
from PIL import Image
from sqlalchemy_utc import UtcDateTime, utcnow
from werkzeug.security import check_password_hash, generate_password_hash
from werkzeug.utils import cached_property

from yadc import db, login, utils

class OP_LEVEL(enum.Enum):
    user = 0
    # creator = 1
    moderator = 5
    admin = 9

class USER_STATUS(enum.Enum):
    active = 0
    # inactive = 1
    banned = 5

class FILETYPE(enum.Enum):
    png = 0
    jpeg = 1

    @property
    def file_ext(self):
        return 'png' if self is FILETYPE.png else 'jpg'

    @property
    def is_jpeg(self):
        return self == FILETYPE.jpeg

class RATING(enum.Enum):
    safe = 0
    questionable = 1
    explicit = 2

    @property
    def matched(self):
        return [r for r in RATING if r.value<=self.value]

class POST_STATUS(enum.Enum):
    pending = 0
    active = 1
    deleted = 2

class TAG_CATEGORY(enum.Enum):
    general = 0
    style = 1
    circle = 2
    artist = 3
    character = 4
    copyright = 5

# class COMMENT_STATUS(enum.Enum):
#     visible = 0
#     deleted = 2

class IMAGE_STORE(enum.Enum):
    image = 'img'
    jpeg = 'jpeg'
    sample = 'sample'
    thumb_big = 'thumb_big'
    thumb_smol = 'thumb'

class TimestampMixin(object):
    created = db.Column(UtcDateTime, nullable=False, server_default=utcnow())
    updated = db.Column(UtcDateTime, nullable=False, onupdate=utcnow(), server_default=utcnow())

    def natural_created(self):
        return humanize.naturaltime(datetime.now().astimezone(self.created.tzinfo) - self.created)

    def natural_updated(self):
        return humanize.naturaltime(datetime.now().astimezone(self.updated.tzinfo) - self.updated)

user_tags_blacklist = db.Table('user_tags_blacklist', db.metadata,
    db.Column('user_id', db.Integer, db.ForeignKey('user.id')),
    db.Column('tag_id', db.Integer, db.ForeignKey('tag.id'))
)

class User(UserMixin, TimestampMixin, db.Model):
    id = db.Column(db.Integer, primary_key=True)
    username = db.Column(db.String(128), unique=True, nullable=False)
    email = db.Column(db.String(256), unique=True)
    pass_hash = db.Column(db.String(128))
    op_level = db.Column(db.Enum(OP_LEVEL, native_enum=False, create_constraint=False), server_default=OP_LEVEL.user.name, nullable=False)
    user_status = db.Column(db.Enum(USER_STATUS, native_enum=False, create_constraint=False), server_default=USER_STATUS.active.name, nullable=False)
    last_login = db.Column(UtcDateTime)

    ban_until = db.Column(UtcDateTime)
    ban_reason = db.Column(db.String(512))

    biography = db.Column(db.String(512))

    rating = db.Column(db.Enum(RATING, native_enum=False, create_constraint=False), server_default=RATING.safe.name, nullable=False)
    tag_blacklist = db.relationship('Tag', secondary=user_tags_blacklist, backref=db.backref('user_blacklisted'))

    def __repr__(self):
        return '<User {}>'.format(self.username)

    def create_password(self, password):
        self.pass_hash = generate_password_hash(password, salt_length=16)

    def check_password(self, password):
        if self.pass_hash is None:
            return False
        return check_password_hash(self.pass_hash, password)

    def login(self, remember):
        login_user(self, remember=remember)
        self.last_login = utcnow()

    def logout(self):
        logout_user()

    @property
    def can_upload(self):
        self.user_status is USER_STATUS.active

    @property
    def is_uploader(self):
        return len(self.authored_posts) > 0

    @property
    def is_moderator(self):
        # return self.op_level in [OP_LEVEL.moderator, OP_LEVEL.admin]
        return self.op_level.value >= OP_LEVEL.moderator.value

    @property
    def is_admin(self):
        return self.op_level is OP_LEVEL.admin

    @property
    def is_current(self):
        return self == current_user


@login.user_loader
def load_user(id):
    return User.query.get(int(id))

def moderator_required(func):
    @wraps(func)
    def dec_view(*args, **kwargs):
        if current_user:
            if not current_user.is_moderator and not current_user.is_admin:
                flash("You don't have sufficient rights to do this.")
                return redirect(url_for('main.index'))
        return func(*args, **kwargs)
    return dec_view

def admin_required(func):
    @wraps(func)
    def dec_view(*args, **kwargs):
        if current_user:
            if not current_user.is_admin:
                flash("You don't have sufficient rights to do this.")
                return redirect(url_for('main.index'))
        return func(*args, **kwargs)
    return dec_view


post_tags = db.Table('post_tags', db.metadata,
    db.Column('post_id', db.Integer, db.ForeignKey('post.id')),
    db.Column('tag_id', db.Integer, db.ForeignKey('tag.id'))
)
    
class Post(TimestampMixin, db.Model):
    id = db.Column(db.Integer, primary_key=True)
    md5 = db.Column(db.String(32), unique=True, nullable=False)
    filetype = db.Column(db.Enum(FILETYPE, native_enum=False, create_constraint=False), nullable=False)
    rating = db.Column(db.Enum(RATING, native_enum=False, create_constraint=False), nullable=False)
    status = db.Column(db.Enum(POST_STATUS, native_enum=False, create_constraint=False), server_default=POST_STATUS.pending.name, nullable=False)

    width = db.Column(db.Integer, default=0)
    height = db.Column(db.Integer, default=0)
    filesize = db.Column(db.Integer, default=0)

    source = db.Column(db.String(2048))
    origin_filename = db.Column(db.String(255))

    author_id = db.Column(db.Integer, db.ForeignKey('user.id'))
    author = db.relationship('User', backref=db.backref('authored_posts', lazy=True), foreign_keys=[author_id])
    approver_id = db.Column(db.Integer, db.ForeignKey('user.id'))
    approver = db.relationship('User', backref=db.backref('approved_posts', lazy=True), foreign_keys=[approver_id])

    tags = db.relationship('Tag', secondary=post_tags, backref=db.backref('posts'))

    parent_id = db.Column(db.Integer, db.ForeignKey('post.id'))
    parent = db.relationship('Post', backref=db.backref('children', lazy=True), remote_side=[id])
    # children = db.relationship('Post', back_populates='parent')

    @property
    def resolution(self):
        return (self.width, self.height)

    @resolution.setter
    def resolution(self, value):
        self.width, self.height = value

    @staticmethod
    def fileinfo(file):
        return dict(
            md5=(file.seek(0) or hashlib.md5(file.read()).hexdigest()),
            filetype=FILETYPE[file.mimetype.split('/')[1]], # not safe
            resolution=Image.open(file).size,
            filesize=(file.seek(0,2) or file.tell()),
            origin_filename=file.filename
        )

    def generate_image_files(self):
        with Image.open(self.image_files['image']) as im:
            im = im.convert('RGB')

            if self.image_files['jpeg'] is not None:
                im.save(self.image_files['jpeg'], 'JPEG', quality=90) # SUM QUALITY, YEEES

            sim = im.copy()
            sim.thumbnail([1500,1500])
            sim.save(self.image_files['sample'], 'JPEG', quality=80)
            
            sim = im.copy()
            sim.thumbnail([800,800])
            sim.save(self.image_files['thumb_big'], 'JPEG', quality=80)

            sim = im.copy()
            sim.thumbnail([512,512])
            sim.save(self.image_files['thumb_smol'], 'JPEG', quality=80)

    def remove_image_files(self):
        for im in self.image_files.values():
            try:
                if im: os.remove(im)
            except:
                pass

    @property
    def image_files(self):
        return dict(
            image=os.path.join(current_app.config.get('POST_UPLOADS'), IMAGE_STORE.image.value, f"{self.md5}.{self.filetype.file_ext}"),
            jpeg=(os.path.join(current_app.config.get('POST_UPLOADS'), IMAGE_STORE.jpeg.value, f"{self.md5}.jpg") if not self.filetype.is_jpeg else None),
            sample=os.path.join(current_app.config.get('POST_UPLOADS'), IMAGE_STORE.sample.value, f"{self.md5}.jpg"),
            thumb_big=os.path.join(current_app.config.get('POST_UPLOADS'), IMAGE_STORE.thumb_big.value, f"{self.md5}.jpg"),
            thumb_smol=os.path.join(current_app.config.get('POST_UPLOADS'), IMAGE_STORE.thumb_smol.value, f"{self.md5}.jpg")
        )

    def url(self, path, endpoint=IMAGE_STORE.image):
        store = {s.name:s for s in IMAGE_STORE}.get(endpoint, IMAGE_STORE.image) if not isinstance(endpoint, IMAGE_STORE) else endpoint
        if store is IMAGE_STORE.image:
            return url_for(
                'main.uploaded_img',
                path=f"{path}.{self.filetype.file_ext}"
            )
        elif store is IMAGE_STORE.jpeg:
            return url_for(
                'main.uploaded_jpeg' if not self.filetype.is_jpeg else 'main.uploaded_img',
                path=f"{path}.jpg"
            )
        elif store is IMAGE_STORE.sample:
            return url_for('main.uploaded_sample', path=f"{path}.jpg")
        elif store is IMAGE_STORE.thumb_big:
            return url_for('main.uploaded_big_thumb', path=f"{path}.jpg")
        elif store is IMAGE_STORE.thumb_smol:
            return url_for('main.uploaded_smol_thumb', path=f"{path}.jpg")

    @property
    def file_uri(self):
        return os.path.join(
            self.md5,
            "{} - {} {}".format(
                current_app.config.get('INSTANCE_NAME'),
                self.id,
                " ".join(tag.content.replace(' ', '_') for tag in self.tags)
            )
        )

    def __repr__(self):
        return('<Post #{} by {}, {} of {} bytes>'.format(self.id, self.author, self.filetype.name, self.filesize))

    @cached_property
    def flex(self):
        return "flex: {0:.2f} 1 {0:.2f}px;".format(self.width/self.height*current_app.config.get('POST_LIST_THUMB_HEIGHT', 240))

    @cached_property
    def image_resolution(self):
        return "{}x{}".format(self.width, self.height)

    @cached_property
    def filesize_human(self):
        return utils.sizeof_fmt(self.filesize)

    @property
    def is_approved(self):
        return self.status is POST_STATUS.active

    @property
    def can_edit(self):
        author = current_user == self.author
        moderator = current_user.is_moderator
        return author or moderator

class Tag(TimestampMixin, db.Model):
    id = db.Column(db.Integer, primary_key=True)
    content = db.Column(db.String(128), unique=True, nullable=False)
    category = db.Column(db.Enum(TAG_CATEGORY, native_enum=False, create_constraint=False), server_default=TAG_CATEGORY.general.name, nullable=False)

    @property
    def content_deser(self):
        return self.content.replace('_',' ')

    @content_deser.setter
    def content_deser(self, value):
        self.content = value.replace(' ', '_')

    def __repr__(self):
        return '<Tag {}, {}>'.format(self.content, self.category.name)

class Comment(TimestampMixin, db.Model):
    id = db.Column(db.Integer, primary_key=True)
    content = db.Column(db.String(512), nullable=False)
    visible = db.Column(db.Boolean, default=True)

    # status = db.Column(db.Enum(COMMENT_STATUS), default=COMMENT_STATUS.visible)
    deleted = db.Column(db.Boolean, default=False)

    post_id = db.Column(db.Integer, db.ForeignKey('post.id'))
    post = db.relationship('Post', backref=db.backref('comments', lazy=True))
    user_id = db.Column(db.Integer, db.ForeignKey('user.id'))
    user = db.relationship('User', backref=db.backref('comments', lazy=True))

    @property
    def can_modify(self):
        # author = current_user == self.user
        moderator = current_user.is_moderator
        return self.is_author or moderator

    @property
    def is_author(self):
        return current_user == self.user